###############################################################################
# 
# Excelǡץ饰
# 
###############################################################################
package plugin::excel::MakeSheet;

use strict;
use plugin::excel::DataParser;
use Spreadsheet::WriteExcel;
use Jcode;

use vars qw($Ocode $Attach %Format);

#==============================================================================
$Ocode  = 'sjis';                   # output kanji code
$Attach = 0;                        # if true, make attachment file
%Format = (                         # font format in cell
	q/'''/ => { bold   => 1 },
	q/''/  => { italic => 1 },
	q/__/  => { underline => 1 },
	q/==/  => { font_strikeout => 1 },
	q/[/   => { underline => 1, color => 'blue' },
	q/[[/  => { underline => 1, color => 'blue' },
	q/1st/ => { bold => 1, align => 'center', bg_color => 'silver' }, # 1st row
);
#==============================================================================
sub new {
	my $class = shift;
	return bless {},$class;
}
#==============================================================================
sub do_action {
	my $self = shift;
	my $wiki = shift;
	my $cgi = $wiki->get_CGI;

	my $pagename = $cgi->param("page");
	my $table_id = $cgi->param("table_id");

	if( my $error = $self->check_param($wiki,$pagename,$table_id) ){
		return $wiki->error($error);
	}

	my $file = $wiki->config('log_dir') . '/' . Util::url_encode($pagename)
	         . (($table_id != 0) ? "-$table_id" : '') . '.xls';

	my $modtime_xls = (stat($file))[9];

	if(!$modtime_xls or $wiki->get_last_modified($pagename) > $modtime_xls){
		$self->make_sheet($wiki,$file,$pagename,$table_id)
		 or return $wiki->error($self->{error});
	}

	print "Content-type: application/vnd.ms-excel\n";
	print "Content-Disposition: attachment; filename=$file\n" if($Attach);
	print "\n";

	open(SHEET,$file);
	binmode(SHEET);
	while(my $line = <SHEET>){ print $line; }
	close(SHEET);

	$wiki->exit(); # before 3.5.2, must be commented out
}


sub make_sheet{
	my $self    = shift;
	my $wiki    = shift;
	my $file    = shift;
	my $source  = $wiki->get_page(shift);
	my $workbook = Spreadsheet::WriteExcel->new($file);

	$self->{table_id} = shift;

	$self->{workbook}  = $workbook;
	$self->{worksheet} = $workbook->add_worksheet();
	$self->{worksheet}->keep_leading_zeros();

	my $parser = plugin::excel::DataParser->new($wiki);
	# status is used for can_ignore(), id_num for checking table_id
	@$self{qw(status id_num col row)} = (0,1,0,0);

	for my $line ( split(/\n/,$source) ){
		# check text lines and set internal status
 		next if( $self->can_ignore($line) );

		my $cols      = $parser->parse($line)->get_result();
		my $format    = $workbook->add_format(border=>1);
		my $worksheet = $self->{worksheet};

		$format->set_properties(%{$Format{'1st'}}) if($self->{row} == 0);
		$self->{format} = $format;

		# make rows
		for my $type ( @$cols ){
			if(!ref $type){
				$self->make_cell();
			}
			elsif(!defined $type->[0]){ # simple text
				$self->make_cell(jcode($type->[1],'euc')->$Ocode(),$format);
			}
			elsif($type->[0] eq '[' or $type->[0] eq '[['){ # anchor link
				my $url  = $type->[1][0];
				my $name = (defined $type->[1][1])
				            ? jcode($type->[1][1],'euc')->$Ocode() : $url;
				my $ft   = $self->make_format($Format{$type->[0]});
				$self->make_cell([$url,$name],$ft);
			}
			else{ # bold, italic and so on
				my $str = jcode($type->[1],'euc')->$Ocode();
				my $ft  = $self->make_format($Format{$type->[0]});
				$self->make_cell($str,$ft);
			}
			$self->{col}++;

		}

	}
	# clean up
	$workbook->close();

	if($self->{is_sheet}){
		return 1;
	}
	else{
		$self->{error} = "table_idλ꤬ޤ";
		unlink($file);
		return 0;
	}
}

sub make_cell{
	my $self   = shift;
	my $data   = shift;
	my $format = shift;

	if(ref $data eq 'ARRAY'){
		$self->{worksheet}->write_url(
		    $self->{row},$self->{col},$data->[0],$data->[1],$format
		);
		$data = $data->[1];
	}
	else{
		$self->{worksheet}->write($self->{row},$self->{col},$data,$format);
	}

	$self->set_col_length($data);
}

sub make_format{
	my $self   = shift;
	my $method = shift;
	my $format  = $self->{workbook}->add_format();
	$format->copy($self->{format});
	$format->set_properties( %$method ) if(defined $method);
	return $format;
}

sub set_col_length{
	my $self = shift;
	my $str  = shift;
	my $len  = length($str);
	my $col  = $self->{col};
	my $id   = $self->{is_sheet} - 1;

	if($len > $self->{col_length}->[$id]->[$col]){
		$self->{worksheet}->set_column($self->{col},$self->{col},$len);
		$self->{col_length}->[$id]->[$col] = $len;
	}
}

sub can_ignore{
	my $self   = shift;
	my $line   = shift;
	my $status = $self->{status};

	if($status == 1 and $line !~ /^,/){ # reset status
		($self->{status},$self->{col},$self->{row}) = (0,0,0);
		$self->{id_num}++;
		return 1;
	}
	elsif($status == 0 and $line !~ /^,/){ # not table format
		return 1;
	}
	elsif($status == 2 and $line !~ /^,/){ # table_id didn't correspond and reset
		$self->{id_num}++; $self->{status} = 0; return 1;
	}
	elsif($status == 2){ # table_id don't correspond
		return 1;
	}
	elsif($status == 0 and $line =~ /^,/){ # table format!
		if($self->{table_id} and $self->{table_id} != $self->{id_num}){
			$self->{status} = 2; return 1; # but table_id didn't correspond
		}
		else{ # ok, we are going to make sheet.
			$self->{status}    = 1; $self->{is_sheet}++;
			$self->{worksheet} = $self->{workbook}->add_worksheet()
			 if($self->{is_sheet} > 1);
			$self->{worksheet}->keep_leading_zeros();
			return 0;
		}
	}
	else{ # go to next row
		$self->{col} = 0; $self->{row}++;
	}

	return 0;
}

sub check_param{ # if any error, return true.
	my $self = shift;
	my ($wiki,$page,$id) = @_;
	my $str;

	$str = "table_idλ꤬Ǥ"     if(!defined $id or $id =~ /\D/);
	$str = "ڡꤵƤޤ"   if(!defined $page);
	$str = "Υڡ¸ߤޤ"   if(!$wiki->page_exists($page));
	$str = "ΥڡϻȤǤޤ" if(!$wiki->can_show($page));

	return $str;
}

1;
